/**
   This program uses an approximation of the
   gradient vector to detect edges in a picture.
*/

import framebuffer.*;

import java.awt.Color;

public class EdgeDetection
{
   public static final double THRESHOLD = 50.0;

   public static void main(String[] args)
   {
      // Check for a file name on the command line.
      if ( 0 == args.length )
      {
         System.err.println("Usage: java Edges <PPM-file-name>");
         System.exit(-1);
      }
      // the input image
      FrameBuffer fb1 = new FrameBuffer( args[0] );
      // the output image
      FrameBuffer fb2 = new FrameBuffer(fb1.width, fb1.height, Color.black);

      for (int y = 1; y < fb1.height - 1; ++y)
      {
         for (int x = 1; x < fb1.width - 1; ++x)
         {
            Color c0 = fb1.getPixelFB(x, y);
            Color cLeft  = fb1.getPixelFB(x, y - 1);
            Color cRight = fb1.getPixelFB(x, y + 1);
            Color cAbove = fb1.getPixelFB(x - 1, y);
            Color cBelow = fb1.getPixelFB(x + 1, y);

            double deltaHr = cRight.getRed() - cLeft.getRed();
            double deltaHg = cRight.getGreen() - cLeft.getGreen();
            double deltaHb = cRight.getBlue() - cLeft.getBlue();
            double deltaH  = Math.max(deltaHr, Math.max(deltaHg, deltaHb));
            double deltaVr = cAbove.getRed() - cBelow.getRed();
            double deltaVg = cAbove.getGreen() - cBelow.getGreen();
            double deltaVb = cAbove.getBlue() - cBelow.getBlue();
            double deltaV  = Math.max(deltaVr, Math.max(deltaVg, deltaVb));
            double grad = Math.pow( deltaH*deltaH + deltaV*deltaV, 0.5 );

            Color cNew = Color.black;
            if ( grad > THRESHOLD ) // found an edge
            {
               cNew = Color.white;
            }
            fb2.setPixelFB( x, y, cNew );
         }
      }
      String baseName = args[0].substring(0, args[0].lastIndexOf("."));
      String savedFileName = baseName + "_edges" + ".ppm";
      fb2.dumpFB2File( savedFileName );
   }
}
